-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

--
-- Function to return the index for any undefined indexes
--
local function defaultValueIndex(table, index)
  return index;
end

-- Set the function for any undefined indexes
setmetatable(L, {__index = defaultValueIndex});

local NUM_INVITES_PER_PAGE = 10;

-------------------------------------------------------------------------------
--                 L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to add a name to a frame's character list
--
function NigglesCharacterListAddName(frame, name)
  -- Add the name to the frame's list
  if ((frame.list ~= nil) and
      (not tContains(frame.list, name))) then
     tinsert(frame.list, name);
     sort(frame.list);
  end

  -- Update the frame
  NigglesCharacterListUpdate(_G[frame:GetName().."Scrollbar"]);

  return;
end

--
-- Function to display drop down menu for character list item
--
function NigglesCharacterListButtonDropDown(button)
  -- Local Variables
  local menuItemInfo =
  {
    {
      text         = button:GetText(),
      isTitle      = "true",
      notCheckable = "true"
    },
    {
      text         = DELETE,
      notCheckable = "true",
      func = function()
        NigglesCharacterListDeleteName(button:GetParent(), button:GetText());
      end
    }
  };

  -- Play a sound
  PlaySound("igMainMenuOptionCheckBoxOn");

  -- Create the drop down menu
  EasyMenu(menuItemInfo, button:GetParent().dropDown, "cursor", 0, 0, "MENU");

  return;
end

--
-- Function to delete a name from a frame's character list
--
function NigglesCharacterListDeleteName(frame, name)
  -- Delete the name from the frame's list
  if (frame.list ~= nil) then
    for key, value in pairs(frame.list) do
      if (value == name) then
        table.remove(frame.list, key);
        break;
      end
    end
  end

  -- Update the frame
  NigglesCharacterListUpdate(_G[frame:GetName().."Scrollbar"]);

  return;
end

--
-- Function to process the Enter key in an Character List edit box
--
function NigglesCharacterListEditOnEnterPressed(self)
  -- Local Variables
  local name;
  local frame = self:GetParent();

  -- Check the AutoComplete frame doesn't want the key press
  if (not AutoCompleteEditBox_OnEnterPressed(self)) then
    -- Get the name entered by the user
    name = strtrim(self:GetText());
    if (name ~= "") then
      -- Capitalize the name
      name = (name:sub(1,1):upper())..(name:sub(2):lower());

      -- Add the name to the list
      NigglesCharacterListAddName(frame, name)

      -- Delete the name from the list of the frame's sibling
      if (frame.sibling ~= nil) then
        NigglesCharacterListDeleteName(frame.sibling, name)
      end

      -- Clear the edit frame
      self:SetText("");
    else
      self:ClearFocus();
    end
  end

  return;
end

--
-- Function to tidy up when a Character List Edit frame loses focus
--
function NigglesCharacterListEditOnFocusLost(self)
  AutoCompleteEditBox_OnEditFocusLost(self);
  self:HighlightText(0, 0);
  local trimmedText = strtrim(self:GetText());
  if (trimmedText == "") then
    self:SetText("");
  end

  return;
end

--
-- Function to update the names listed in an Calendar Events Invites list
--
function NigglesCharacterListUpdate(scrollbar)
  -- Local Variables
  local button;
  local frameName;
  local frame = scrollbar:GetParent();
  local numNames = 0;
  local topLine;

  -- Check the required variables are initialised
  if ((frame ~= nil) and (frame.list ~= nil)) then
    -- Get the frame's name
    frameName = frame:GetName();

    -- Get the number of names in the list
    numNames = #(frame.list);

    -- Make sure the top line displayed is still valid
    topLine = FauxScrollFrame_GetOffset(scrollbar);
    if (topLine+NUM_INVITES_PER_PAGE > numNames) then
      -- Work out the new top line
      topLine = numNames-NUM_INVITES_PER_PAGE;
      if (topLine < 0) then
        topLine = 0;
      end

      -- Set the new top line
      FauxScrollFrame_SetOffset(scrollbar, topLine);
    end

    -- Update all the buttons
    for buttonIdx = 1, NUM_INVITES_PER_PAGE do
      -- Get the current button
      button = _G[frameName..buttonIdx];

      -- Work out the index of the invite to display in the button
      if (topLine+buttonIdx <= numNames) then
        button:SetText(frame.list[topLine+buttonIdx]);
        button:Show();
        button:Enable();
      else
        button:Hide();
        button:Disable();
      end
    end

    -- Update the scrollbar
    FauxScrollFrame_Update(scrollbar, numNames, NUM_INVITES_PER_PAGE, 14,
      nil, 0, 0, nil, 0, 0, true);
  end

  return;
end

--
-- Function to enable/disable a frame and all its descendants. This function
-- is recursive.
--
function NigglesEnableFrame(frame, enable)
  -- Local Variables
  local childType;
  local frameType;

  -- Enable/disable the frame, if possible
  if ((frame.Enable) and (enable == true)) then
    frame:Enable();
  elseif ((frame.Disable) and (enable == false)) then
    frame:Disable();
  elseif (frame.GetObjectType) then
    frameType = frame:GetObjectType();
    if (frameType == "Frame") then
      -- Process each region
      for _, child in ipairs({frame:GetRegions()}) do
        -- Get the child's type
        childType = child:GetObjectType();

        -- Process the child, based on its type
        if (childType == "FontString") then
          if ((child.normalFont ~= nil) and (enable == true)) then
            child:SetFontObject(child.normalFont);
          elseif ((child.disableFont ~= nil) and (enable == false)) then
            child:SetFontObject(child.disableFont);
          end
        end
      end
    elseif (frameType == "EditBox") then
      frame:EnableJoystick(enable);
      frame:EnableMouse(enable);
      frame:EnableMouseWheel(enable);
      frame:EnableKeyboard(enable);
      frame:ClearFocus();
      if ((frame.normalFont ~= nil) and (enable == true)) then
        frame:SetFontObject(frame.normalFont);
      elseif ((frame.disableFont ~= nil) and (enable == false)) then
        frame:SetFontObject(frame.disableFont);
      end
    end
  end

  -- Process all children of the current frame
  for _, child in ipairs({frame:GetChildren()}) do
    NigglesEnableFrame(child, enable);
  end

  return;
end

--
-- Function to get the localized version of a string
--
function NigglesGetLocaleString(string)
  return L[string];
end

--
-- Function to check if a value is in a table, ignoring case
--
function NigglesTableContains(table, value)
  -- Local Variables
  local idx = 1;

  -- Search the table for the specified value
  value = value:lower();
  while (table[idx]) do
    if (value == table[idx]:lower()) then
      return 1;
    end
    idx = idx+1;
  end

  return nil;
end

--
-- Function to create a timer
--
function NigglesTimerCreate(name, func, arg1, arg2, delay, repeating)
  -- Local Variables
  local currentTimer;

  -- Search for the named timer
  currentTimer = Niggles.timers.list;
  while (currentTimer ~= nil) do
    -- Check if the timer is the one specified
    if (currentTimer.name == name) then
      break;
    end
    currentTimer = currentTimer.next;
  end

  -- Check if the timer already exists
  if (currentTimer ~= nil) then
    -- Re-initialise the existing timer
    currentTimer.func      = func;
    currentTimer.arg1      = arg1;
    currentTimer.arg2      = arg2;
    currentTimer.delay     = delay;
    currentTimer.repeating = repeating;
    currentTimer.elapsed   = 0;
    currentTimer.delete    = false;
  else
    -- Create the new timer
    local newTimer = {};
    newTimer.name      = name;
    newTimer.func      = func;
    newTimer.arg1      = arg1;
    newTimer.arg2      = arg2;
    newTimer.delay     = delay;
    newTimer.repeating = repeating;
    newTimer.elapsed   = 0;
    newTimer.delete    = false;
    newTimer.prev      = nil;
    newTimer.next      = nil;

    -- Add the new timer to the linked list
    if (Niggles.timers.list == nil) then
      Niggles.timers.list = newTimer;
    else
      Niggles.timers.list.prev = newTimer;
      newTimer.next            = Niggles.timers.list;
      Niggles.timers.list      = newTimer;
    end
  end

  -- Start the monitoring loop, if required
  if (Niggles:GetScript("OnUpdate") == nil) then
    Niggles:SetScript("OnUpdate", NigglesOnUpdate);
  end

  return;
end

--
-- Function to destroy a timer
--
function NigglesTimerDestroy(name)
  -- Local Variables
  local currentTimer = Niggles.timers.list;

  -- Search for the named timer
  while (currentTimer ~= nil) do
    -- Check if the timer is the one specified
    if (currentTimer.name == name) then
      -- Expire the timer and mark it for deletion
      currentTimer.elapsed   = 1;
      currentTimer.delay     = 0;
      currentTimer.repeating = false;
      currentTimer.delete    = true;
      break;
    end

    -- Move onto the next timer
    currentTimer = currentTimer.next;
  end

  return;
end

--
-- Function called each time the Niggles frame is redrawn to the screen.
-- It is used to do periodic tasks
--
function NigglesOnUpdate(self, elapsed)
  -- Update the master elapse timer
  Niggles.timers.elapsed = Niggles.timers.elapsed+elapsed;

  -- Check if the master elapsed timer has expired
  if (Niggles.timers.elapsed > 0.100) then
    -- Local Variables
    local currentTimer;
    local nextTimer;

    -- Process the Niggles.timers
    currentTimer = Niggles.timers.list;
    while (currentTimer ~= nil) do
      -- Update the timer's elapsed time
      currentTimer.elapsed = currentTimer.elapsed+Niggles.timers.elapsed;

      -- Save the next timer
      nextTimer = currentTimer.next;

      -- Check if the timer has expired
      if (currentTimer.elapsed >= currentTimer.delay) then
        -- Call the timer's function
        if (currentTimer.delete ~= true) then
          currentTimer.func(currentTimer.arg1, currentTimer.arg2);
        end

        -- Check if the timer is a repeating timer
        if (currentTimer.repeating == true) then
          -- Reset the timer
          currentTimer.elapsed = 0;
        else
          -- Remove the timer from the list
          if (currentTimer.prev ~= nil) then
            currentTimer.prev.next = currentTimer.next;
          else
            Niggles.timers.list = currentTimer.next;
          end
          if (currentTimer.next ~= nil) then
            currentTimer.next.prev = currentTimer.prev;
          end
          currentTimer = nil;
        end
      end

      -- Move onto the next timer
      currentTimer = nextTimer;
    end

    -- Reset the master elapse timer
    Niggles.timers.elapsed = 0;
  end

  -- Stop the monitoring loop, if no longer required
  if (Niggles.timers.list == nil) then
    Niggles:SetScript("OnUpdate", nil);
  end

  return;
end
