-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local NUM_CRITTERS_PER_ROW  = 6;
local NUM_CRITTERS_PER_PAGE = 12;

-- Used to build a mask for summoning critters
local critterMasks =
{
  ["Kalimdor"]         = 0x0001,
  ["Eastern Kingdoms"] = 0x0002,
  ["Outland"]          = 0x0004,
  ["Northrend"]        = 0x0008,
  ["city"]             = 0x0010,
  ["party"]            = 0x0020,
  ["raid"]             = 0x0040,
  ["arena"]            = 0x0080,
  ["pvp"]              = 0x0100,
};

-- Critters used as part of a quest
local questCritters =
{
  -- [31880] = -- Blood Elf Orphan Whistle
  {
    "all"
  },
  [31881] = -- Draenei Orphan Whistle
  {
    "all"
  },
  [30803] = -- Felhound Whistle
  {
    L["Hellfire Peninsula"]
  },
  [36865] = -- Golem Control Unit
  {
    L["Dun Argol"]
  },
  [36865] = -- Golem Control Unit
  {
    "all"
  },
  [18598] = -- Human Orphan Whistle
  {
    "all"
  },
  [46397] = -- Oracle Orphan Whistle
  {
    "all"
  },
  [18597] = -- Orcish Orphan Whistle
  {
    "all"
  },
  [34253] = -- Sizzling Ember
  {
    L["Throne of Kil'jaeden"]
  },
  [46362] =
  {
    L["Silithus"],
    L["Tanaris"],
    L["Un'Goro Crater"]
  },
  [68646] = -- Winterspring Cub
  {
    L["Winterspring"]
  },
  [46396] = -- Wolvar Orphan Whistle
  {
    "all"
  },
  [31815] = -- Zeppit's Crystal
  {
    L["Netherstorm"]
  }
};

local restrictedZones = {};

local cityZones =
{
  [L["Dalaran"]]         = true;
  [L["Darnassus"]]       = true;
  [L["Ironforge"]]       = true;
  [L["Orgrimmar"]]       = true;
  [L["Shattrath City"]]  = true;
  [L["Silvermoon City"]] = true;
  [L["Stormwind"]]       = true;
  [L["The Exodar"]]      = true;
  [L["Thunderbluff"]]    = true;
  [L["Undercity"]]       = true;
};

-------------------------------------------------------------------------------
--                L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function to process clicks on check buttons in the Niggles Options
-- Companions panel.
--
function NigglesCritterClicked(checkButton)
  -- Check the button has the required info
  if ((checkButton ~= nil) and
      (checkButton.spellId ~= nil)) then
    Niggles.updatedOptions.critters[checkButton.spellId] =
      checkButton:GetChecked();
  end

  return;
end

--
-- Function called when the contents of the player's bags changes. It searches
-- for any quest items used to summon critters
--
function NigglesCrittersBagUpdate()
  -- Clear the list of restricted zones
  wipe(restrictedZones);

  -- Search for any items used to summon quest critters
  for itemId, itemZones in pairs(questCritters) do
    if (GetItemCount(itemId, false, false) > 0) then
      if (itemZones[1] == "all") then
        restrictedZones["all"] = true;
        break;
      else
        for _, zoneName in ipairs(itemZones) do
          restrictedZones[zoneName] = true;
        end
      end
    end
  end

  return;
end

--
-- Function to scroll the critters
--
function NigglesCrittersListScroll(frame)
  -- Update the list
  NigglesCrittersListUpdate(frame);

  return;
end

--
-- Function to update the critters displayed by a frame in the Interface
-- Options 'Companions' panel.
--
function NigglesCrittersListUpdate(frame)
  -- Local Variables
  local button;
  local frameName;
  local icon;
  local numCritters;
  local scrollbar;
  local topLine;

  -- Check the required variables are initialised
  if ((Niggles.updatedOptions ~= nil) and
      (Niggles.updatedOptions.critters ~= nil)) then
    -- Get the frame's name
    frameName = frame:GetName();

    -- Get the number of critters
    numCritters = GetNumCompanions("CRITTER");

    -- Get the frame's scrollbar
    scrollbar = _G[frameName.."Scrollbar"];

    -- Make sure the top line displayed is valid
    topLine = FauxScrollFrame_GetOffset(scrollbar);
    if (topLine > math.ceil(numCritters/NUM_CRITTERS_PER_ROW)) then
      -- Work out the new top line
      topLine = math.ceil(numCritters/NUM_CRITTERS_PER_ROW)-1;
      if (topLine < 0) then
        topLine = 0;
      end

      -- Set the new top line
      FauxScrollFrame_SetOffset(scrollbar, topLine);
    end

    -- Update all the buttons
    for buttonIdx = 1, NUM_CRITTERS_PER_PAGE do
      -- Get the current button
      button = _G[frameName..buttonIdx];

      -- Work out the index of the critter to display in the button
      critterIdx = (topLine*NUM_CRITTERS_PER_ROW)+buttonIdx;
      if (critterIdx <= numCritters) then
        -- Update the button
        button.creatureId, button.creatureName, button.spellId, icon =
          GetCompanionInfo("CRITTER", critterIdx);
        button:SetNormalTexture(icon);
        if (Niggles.updatedOptions.critters[button.spellId] ~= nil) then
          button:SetChecked(true);
        else
          button:SetChecked(false);
        end
        button:Enable();
      else
        -- Update the button
        button:SetChecked(false);
        button:Disable();
      end
    end

    -- Update the scrollbar
    FauxScrollFrame_Update(scrollbar,
      math.ceil(numCritters/NUM_CRITTERS_PER_ROW), 2, 37,
      nil, 0, 0, nil, 0, 0, true);
  end

  return;
end

--
-- Function to set the mask used to determine if critters should be summoned
--
function NigglesCrittersSetMask()
  -- Reset the mask
  Niggles.critterMask = 0;

  -- Add flags for the normal zones
  if (NigglesDBC.settings.CrittersInKalimdor == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask,
      critterMasks["Kalimdor"]);
  end
  if (NigglesDBC.settings.CrittersInEKingdoms == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask,
      critterMasks["Eastern Kingdoms"]);
  end
  if (NigglesDBC.settings.CrittersInNorthrend == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask,
      critterMasks["Northrend"]);
  end
  if (NigglesDBC.settings.CrittersInOutland == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask,
      critterMasks["Outland"]);
  end

  -- Add flags for special types of zone
  if (NigglesDBC.settings.CrittersInCities == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["city"]);
  end
  if (NigglesDBC.settings.CrittersInDungeons == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["party"]);
  end
  if (NigglesDBC.settings.CrittersInRaids == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["raid"]);
  end
  if (NigglesDBC.settings.CrittersInArena == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["arena"]);
  end
  if (NigglesDBC.settings.CrittersInBG == true) then
    Niggles.critterMask = bit.bor(Niggles.critterMask, critterMasks["pvp"]);
  end

  return;
end

--
-- Function to summon a random critter, if appropriate
--
function NigglesCritterSummon()
  -- Local Variables
  local critters = {};
  local summonCritter = true;
  local zoneContinent = "";
  local zoneMask = 0;
  local zoneName;
  local zoneType;

  -- Check if the previously summoned critter is still active
  if (Niggles.critterIdx > 0) then
    local _, _, _, _, isActive = GetCompanionInfo("CRITTER",
      Niggles.critterIdx);
    if (isActive == 1) then
      summonCritter = false;
    end
  end

  -- Get info for the zone the player is currently in
  if (summonCritter == true) then
    zoneName, zoneType = GetInstanceInfo();
    if (zoneType == "none") then
      zoneContinent = zoneName;
      zoneName      = GetRealZoneText();
      if (NigglesIsCity(zoneName) == true) then
        zoneMask = critterMasks["city"];
      elseif (critterMasks[zoneContinent] ~= nil) then
        zoneMask = critterMasks[zoneContinent];
      else
        summonCritter = false;
      end
    else
      if (critterMasks[zoneType] ~= nil) then
        zoneMask = critterMasks[zoneType];
      else
        summonCritter = false;
      end
    end
  end

  -- Check if any special critters might be summoned in the current area
  if ((summonCritter == true) and
      (next(restrictedZones) ~= nil)) then
    if ((restrictedZones["all"] == true) or
        (restrictedZones[zoneName] == true) or
        (restrictedZones[GetSubZoneText()] == true)) then
      summonCritter = false;
    end
  end

  -- Check if a critter needs to be summoned and can be
  if ((summonCritter == true) and
      (bit.band(Niggles.critterMask, zoneMask) == zoneMask) and
      (next(NigglesDBC.critters) ~= nil) and
      (IsMounted() == nil) and
      (IsFlying() == nil) and
      (IsFalling() == nil) and
      (IsStealthed() == nil) and
      (GetNumLootItems() == 0) and
      (UnitInVehicle("player") == nil) and
      (UnitUsingVehicle("player") == nil) and
      (UnitIsDeadOrGhost("player") == nil) and
      (UnitIsCharmed("player") == nil) and
      (UnitIsFeignDeath("player") == nil) and
      (UnitIsPossessed("player") == nil) and
      (UnitCastingInfo("player") == nil) and
      (UnitChannelInfo("player") == nil) and
      (UnitAura("player", L["Food"]) == nil) and
      (UnitAura("player", L["Drink"]) == nil) and
      (UnitAura("player", L["Feign Death"]) == nil) and
      (UnitAura("player", L["Camouflage"]) == nil) and
      (UnitAura("player", L["Herbouflage"]) == nil) and
      (UnitAura("player", L["Spirit of Redemption"]) == nil) and
      (GetSubZoneText() ~= L["Dun Argol"]) and
      (GetSubZoneText() ~= L["Throne of Kil'jaeden"])) then
    -- Get the number of critters known by the player
    numCritters = GetNumCompanions("CRITTER");

    -- Work out which critters have been selected by the player
    for index = 1, numCritters do
      local _, _, spellId, _, isActive = GetCompanionInfo("CRITTER", index);
      if (NigglesDBC.critters[spellId] == 1) then
        critters[1+#critters] = index;
      end
      if (isActive == 1) then
        summonCritter = false;
        break;
      end
    end

    -- Call a random critter, if one isn't already summoned
    if ((summonCritter == true) and
        (next(critters) ~= nil)) then
      Niggles.critterIdx = math.random(#critters);
      CallCompanion("CRITTER", critters[Niggles.critterIdx]);
    end
  end

  return;
end

--
-- Function to check if a specified zone is a city
--
function NigglesIsCity(zoneName)
  return (cityZones[zoneName] ~= nil);
end

--
-- Function to refresh the settings for the Interface Options 'Companions'
-- panel
--
function NigglesOptionsRefreshCritters()
  -- Initialise all the widgets to the correct state for the settings
  NigglesOptionsRefreshSettings(NigglesOptionsCritters);

  -- Set the scroll frames to the top
  FauxScrollFrame_OnVerticalScroll(
    NigglesOptionsCrittersCompanionsScrollbar,  0, 37, nil);
  NigglesCrittersListUpdate(NigglesOptionsCrittersCompanions);

  return;
end
